from flask import Flask, g
import utils
from frontend.views import mod as main_views
from metrics import MetricsThread
from frontend.utils import MyJsonEncoder
import wtforms_json
import services
import logging
import os


def create_app():
    app = Flask(__name__)
    if os.getenv('TAPPERUI_DEBUG', False):
        app.config.from_object('fmtapper_web.config.DevelopmentConfig')
    else:
        app.config.from_object('fmtapper_web.config.Config')

    app.config.from_envvar('FMTAPPER_WEB_SETTINGS', silent=True)
    app.config.setdefault('INSTALLATION_ID', services.get_installation_id(app.config['INSTALLATION_ID_FILE']))
    app.secret_key = '\xa0T\xb6\x8d\xfex\x1c\xa0TUE\xd0\xc6\xee\xff\x83fmmv\x8f\xd5[\xc5'

    if not app.debug:
        app.logger.setLevel(logging.DEBUG)
        del app.logger.handlers[:]
        handler = logging.StreamHandler()
        handler.setFormatter(logging.Formatter("[%(asctime)s] %(message)s", datefmt="%Y-%m-%d %H:%M:%S"))
        app.logger.addHandler(handler)

    init_db(app)

    def start_metrics():
        t = MetricsThread(app.config['DATABASE'], app.logger)
        t.start()

    def before_request():
        get_db(app.config['DATABASE'])

    app.before_request(before_request)
    app.add_template_filter(format_ram, "format_ram")
    app.teardown_appcontext(teardown)

    app.register_blueprint(main_views)
    app.before_first_request(start_metrics)
    app.json_encoder = MyJsonEncoder

#   init wtforms-json
    wtforms_json.init()

    print 'Created app'
    return app


def format_ram(v):
    return u'{0:.2f}'.format((1.0*v)/1024/1024/1024)


def get_db(database):
    db = getattr(g, '_database', None)
    if db is None:
        db = g._database = utils.connect_to_db(database)
    return db


def init_db(app):
    with app.app_context():
        db = get_db(app.config['DATABASE'])
        with app.open_resource('schema.sql', mode='r') as f:
            db.cursor().executescript(f.read())
        db.commit()


def teardown(exception):
    db = getattr(g, '_database', None)
    if db is not None:
        db.close()
