# *-* coding: utf-8 *-*
from ConfigParser import NoOptionError

from flask import current_app
import datetime
from errors import ConfigError
import os
from ConfigParser import ConfigParser
import time


class FakeGlobalSectionHead(object):
    SECTION = '[global]\n'

    def __init__(self, fp):
        self.fp = fp
        self.section = FakeGlobalSectionHead.SECTION

    def readline(self):
        if self.section:
            try:
                return self.section
            finally:
                self.section = None
        else:
            return self.fp.readline()

    def write(self, line):
        if line == FakeGlobalSectionHead.SECTION:
            pass
        else:
            self.fp.write(line)


class DefaultValueConfigParser(object):
    def __init__(self, parser):
        self.parser = parser

    def get(self, section, option, value=None, optional=False):
        try:
            return self.parser.get(section, option)
        except NoOptionError, e:
            if value or optional:
                return value
            else:
                DefaultValueConfigParser._handle_no_option_error(option, e)

    def getint(self, section, option, value=None, optional=False):
        try:
            return self.parser.getint(section, option)
        except NoOptionError, e:
            if value or optional:
                return value
            else:
                DefaultValueConfigParser._handle_no_option_error(option, e)

    def getboolean(self, section, option, value=None, optional=False):
        try:
            return self.parser.getboolean(section, option)
        except NoOptionError, e:
            if value or optional:
                return value
            else:
                DefaultValueConfigParser._handle_no_option_error(option, e)

    @staticmethod
    def _handle_no_option_error(option, error):
        current_app.logger.error('Option not configured: %s', option, exc_info=error)
        raise ConfigError('Option not configured: %s' % option)


def get_config_dir():
    return current_app.config['FMTAPPER_CONFIG_DIR']


def get_profiles_dir():
    """
    Возвращает путь каталога настроек тюнеров
    :return:
    """
    return current_app.config['FMTAPPER_PROFILES_DIR']


def get_tuner_dir(tuner_id):
    """
    Возвращает путь каталога настроек тюнера
    :param tuner_id: Id тюнера
    :return:
    """
    return os.path.join(get_profiles_dir(), tuner_id)


def get_schedules_dir(tuner_id):
    return os.path.join(get_tuner_dir(tuner_id), 'schedules')


def get_state_dir():
    return current_app.config['FMTAPPER_STATE_DIR']


def get_logs_dir():
    return current_app.config['FMTAPPER_LOGS_DIR']


def get_installation_id_file():
    return current_app.config['INSTALLATION_ID_FILE']


def get_installation_id():
    return current_app.config['INSTALLATION_ID']


def get_new_tuner_tmp_dir():
    return current_app.config['NEW_TUNER_TMP_DIR']


def get_current_time():
    dt = datetime.datetime.now()
    return time.mktime(dt.timetuple())-time.timezone


def get_password():
    return current_app.config['PASSWORD']


def get_tuner_socket_file(tuner_id):
    """
    Возвращает полный путь файла сокета тюнера
    :param tuner_id: Id тюнера
    :return:
    """
    return os.path.join(current_app.config['FMTAPPER_SOCKET_DIR'], tuner_id + '.sock')


TUNER_PROFILE_CONFIG_FILE_NAME = 'fmtapper.conf'


TUNER_CONFIG_FILE_NAME = 'tapperui.conf'

TUNER_LOGGING_CONFIG_FILE_NAME = 'logging.conf'

TUNER_FEED_CONFIG_FILE_NAME = 'feed.conf'


def get_config_file_path(tuner_id):
    """
    Возвращает полный путь файла конфигурации тюнера
    :param tuner_id: Id тюнера
    :return:
    """
    return os.path.join(get_profiles_dir(), tuner_id, TUNER_CONFIG_FILE_NAME)


def get_profile_file_path(tuner_id):
    """
    Возвращает полный путь файла профиля тюнера
    :param tuner_id: Id тюнера
    :return:
    """
    return os.path.join(get_profiles_dir(), tuner_id, TUNER_PROFILE_CONFIG_FILE_NAME)


def get_schedules_dir_name():
    """
    Возвращает имя каталога хранения файлов расписаний
    :return:
    """
    return current_app.config['FMTAPPER_SCHEDULES_DIR_NAME']


def get_cleanup_service_config_path():
    """
    Возвращает путь к файлу настроек сервиса очистки
    :return:
    """
    return current_app.config['FMTAPPER_CLEANUP_CONFIG_FILE']


def get_parser(path):
    with open(path) as fp:
        (basename, filename) = os.path.split(path)
        parser = ConfigParser()
        parser.readfp(FakeGlobalSectionHead(fp), filename)
        return DefaultValueConfigParser(parser)


def get_tuner_logs_dir(tuner_id):
    """
    Возвращает путь каталога логов тюнера
    :param tuner_id: Id тюнера
    :return:
    """
    return os.path.join(current_app.config['FMTAPPER_LOGS_DIR'], tuner_id)


def get_tuner_logging_config_path(tuner_id):
    return os.path.join(get_tuner_dir(tuner_id), TUNER_LOGGING_CONFIG_FILE_NAME)


def get_users_file():
    return current_app.config['USERS_FILE']


def get_cleanup_service_logs_dir():
    """
    Возвращает путь каталога логов сервиса очистки
    :return:
    """
    return os.path.join(current_app.config['CLEANUP_SERVICE_LOGS_DIR'])


def get_cleanup_service_log_pattern():
    return current_app.config['CLEANUP_SERVICE_LOG_PATTERN']


def get_signal_quality_levels():
    return current_app.config['SIGNAL_QUALITY_LEVELS']


def get_signal_volume_levels():
    return current_app.config['SIGNAL_VOLUME_LEVELS']


def get_feed_config_file_path(tuner_id):
    return os.path.join(get_tuner_dir(tuner_id), TUNER_FEED_CONFIG_FILE_NAME)


def get_tmp_feed_config_file_path():
    return os.path.join(get_new_tuner_tmp_dir(), TUNER_FEED_CONFIG_FILE_NAME)


def get_tmp_dir(tuner_id):
    return os.path.join('/tmp/tapperui', tuner_id)


def get_schedules_applied_file_path(tuner_id):
    return os.path.join('/tmp/tapperui', tuner_id, 'schedules_applied')
