# -*- coding: utf-8 -*-

from functools import wraps
from flask import session, request, redirect, url_for, jsonify
from utils import ApiErrorResponse

def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'username' not in session:
            return redirect(url_for('.login', next=request.url))
        return f(*args, **kwargs)
    return decorated_function


def api_login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'username' not in session:
            return jsonify(*ApiErrorResponse(global_errors='Необходимо авторизоваться', status=403, field_errors=[]))
        return f(*args, **kwargs)
    return decorated_function

def admin_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'username' not in session or not session['username'] == 'admin':
            return redirect(url_for('.login', next=request.url))
        return f(*args, **kwargs)
    return decorated_function


def api_admin_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'username' not in session or not session['username'] == 'admin':
            return jsonify(*ApiErrorResponse(global_errors='Недостаточно прав', status=403, field_errors=[]))
        return f(*args, **kwargs)
    return decorated_function
