import threading
import utils
import psutil
import time
from services import get_tuners


class MetricsThread(threading.Thread):

    def __init__(self, database, logger):
        self.logger = logger
        threading.Thread.__init__(self, name='MetricsThread')
        logger.debug('Created metrics thread')
        self.database = database

    def run(self):
        self.logger.debug('Started metrics thread')
        while True:
            try:
                self.get_metrics()
                self.clear_old_metrics()
                self.check_tuners()
                time.sleep(60)
            except Exception, e:
                print e

    def check_tuners(self):
        for tuner in get_tuners():
            self.logger.debug('Checking tuner %s', tuner.id)

    def get_metrics(self):
        cpu = psutil.cpu_percent(3)
        ram = psutil.virtual_memory().percent
        db = None
        query = 'insert into metrics (ts, cpu, ram) values(?, ?, ?)'
        ts = int(time.time())
#        print '%f %d %d\n' % (time.time(), cpu, ram)
        try:
            db = utils.connect_to_db(self.database)
            db.execute(query, [ts, cpu, ram])
            db.commit()
        except Exception, e:
            self.logger.error(e)
        finally:
            if db is not None:
                db.close()

    def clear_old_metrics(self):
        query = 'delete from metrics where ts < ?'
        ts = int(time.time()) - 115200
        try:
            db = utils.connect_to_db(self.database)
            db.execute(query, [ts])
            db.commit()
        except Exception, e:
            self.logger.error(e)
        finally:
            if db is not None:
                db.close()
