'use strict';

angular.module('fmtapper.directives', [])
.directive('intervalPicker', function($parse) {
    return {
        restrict: 'E',
        scope: true,
        templateUrl: '/static/js/app/partials/intervalpicker.html',
        link: function(scope, element, attrs) {
            var ngModel = attrs['ngModel'];

            initField(element.find('.start'), 'start');
            initField(element.find('.end'), 'end');

            updateView();

            function str2sec(str) {
                var re = /(\d\d):(\d\d):(\d\d)/;
                var ar = re.exec(str);
                var valid = false;
                var sec = 0;
                if(ar && ar.length > 0) {
                    sec = 3600 * ar[1] + 60 * ar[2] + 1*ar[3];
                    valid = sec >= 0 && sec <= 86400
                        && ar[1] >=0 && ar[1] <= 24
                        && ar[2] >= 0 && ar[2] <= 59
                        && ar[3] >= 0 && ar[3] <= 59;
                }
                else {
                    sec = 0;
                    valid = false;
                }
                return [sec, valid];
            }

            function sec2str(sec) {
                var hours =  Math.floor(sec / 3600);
                var minutes = Math.floor((sec - hours * 3600) / 60);
                var seconds = (sec - hours * 3600 - minutes * 60);
                return (hours < 9 ? '0' + hours : '' + hours) + ':'
                    + (minutes < 9 ? '0' + minutes : minutes) + ':'
                    + (seconds < 9 ? '0' + seconds : seconds);
            }

            function updateView() {
                var start = scope.interval.start;
                var end = scope.interval.end;
                element.find('.start').val(sec2str(start)).toggleClass('has-error', start < 0 || start > 86400);
                element.find('.end').val(sec2str(end)).toggleClass('has-error', end < 0 || end > 86400);
            }

            function initField(el, field) {
                var updateModel = function(el, value) {
                    var res = str2sec(value)
                    scope.interval[field] = res[0];
                    el.toggleClass('has-error', !res[1]);
                };

                el.on('change', function() {
                    updateModel(el, this.value);
                }).on('blur', function() {
                    updateModel(el, this.value);
                })
            }
        }
    }
})
.directive('audioPlayer', function($parse) {
    return {
        restrict: 'E',
        scope: {path:  '='},
        template: '<audio controls preload="none"></audio>',
        link: function(scope, element, attrs) {
            var ngModel = attrs['ngModel'];

            initElement(element, scope);

            function initElement(el, scope) {
                var audio = el.find('audio')[0];
                scope.$watch("path", function() {
                    var src = scope.path;
                    audio.preload = src ? 'audio' : 'none';
                    audio.src = scope.path;
                    if(src) {
                        //audio.play();
                    }
                });

                audio.on('pause', function() {
                    $(this).attr('preload', 'none')
                        .attr('src', '');
                    this.load();
                    $(this).attr('src', scope.path);
                });
            }
        }
    }
})
.directive('broadcastStatus', function($parse) {
    return {
        restrict: 'E',
        scope: {rt:  '='},
        template: '<span></span>',
        link: function(scope, element, attrs) {
            var ngModel = attrs['ngModel'];

            initElement(element, scope);

            function initElement(el, scope) {
                var span = el.find('span');
                scope.$watch("rt", function() {
                    var msg;
                    var rt = scope.rt;
                    if(rt) {
                        if (rt.status == 0) {
                            switch (rt.is_broadcasting) {
                                case 0:
                                    msg = 'Включена';
                                    break;
                                case 1:
                                    msg = 'Выключена';
                                    break;
                                default:
                                    msg = 'Статус неизвестен';
                                    break;
                            }
                        }
                        else {
                            msg = '&nbsp;&nbsp;&#x0336;'
                        }
                        span.html(msg);
                    }
                });
            }
        }
    }
})
.directive('writingStatus', function($parse) {
    return {
        restrict: 'E',
        scope: {rt:  '='},
        template: '<span></span>',
        link: function(scope, element, attrs) {
            var ngModel = attrs['ngModel'];

            initElement(element, scope);

            function initElement(el, scope) {
                var span = el.find('span');
                scope.$watch("rt", function() {
                    var msg;
                    var rt = scope.rt;
                    if(rt) {
                        if (rt.status == 0) {
                            switch (rt.is_writing) {
                                case 0:
                                    msg = 'Активна';
                                    break;
                                case 1:
                                    msg = 'Не активна';
                                    break;
                                default:
                                    msg = 'Статус неизвестен';
                            }
                        }
                        else {
                            msg = '&nbsp;&nbsp;&#x0336;'
                        }
                        span.html(msg);
                    }
                });
            }
        }
    }
})
.directive('signalQuality', function($parse) {
    return {
        restrict: 'E',
        scope: {rt:  '='},
        template: '<span></span>',
        link: function(scope, element, attrs) {
            var ngModel = attrs['ngModel'];

            initElement(element, scope);

            function initElement(el, scope) {
                var span = el.find('span');
                scope.$watch("rt", function() {
                    var html;
                    var rt = scope.rt;
                    if(rt) {
                        if (rt.status == 0) {
                            var quality = !rt.quality || rt.quality < 0 ? 0 : rt.quality;
                            html = '<img src="/static/img/level' + quality + '.png" title="' + rt.quality_abs + '" />';
                        }
                        else {
                            html = '&nbsp;&nbsp;&#x0336;'
                        }
                        span.html(html);
                    }
                });
            }
        }
    }
})
.directive('soundVolume', function($parse, $filter) {
    return {
        restrict: 'E',
        scope: {rt: '='},
        template: '<span></span>',
        link: function (scope, element, attrs) {
            var ngModel = attrs['ngModel'];

            initElement(element, scope);

            function initElement(el, scope) {
                var span = el.find('span');
                var toNumber = $filter('number');
                scope.$watch("rt", function () {
                    var html;
                    var rt = scope.rt;
                    if (rt) {
                        if (rt.status == 0) {
                            var volume = !rt.volume || rt.volume < 0 ? 0 : rt.volume;
                            html = '<img src="/static/img/volume' + volume + '.png" title="'
                                + toNumber(rt.volume_abs, 2) + '" />';
                        }
                        else {
                            html = '&nbsp;&nbsp;&#x0336;'
                        }
                        span.html(html);
                    }
                });
            }
        }
    }
})
.directive('jplayer', function($parse) {
    return  {
        restrict: 'E',
        scope: {path: '=', fnum: '@', autoplay: '@'},
        templateUrl: '/static/js/app/partials/jplayer.html',
        link: function(scope, element, attrs) {
            initElement(element, scope);

            function initElement(el, scope) {
                var containerId = 'jp_container_' + scope.fnum;
                var createPlayer = function(path) {
                    el.find('.jp_container').attr('id', containerId);

                    var player = el.find('.jplayer');
                    player.jPlayer({
                    ready: function(event) {
                        player.jPlayer('setMedia', {
                            title: 'dddd',
                            mp3: path
                        })
                    },
                     swfPath: './',
                     solution: 'html, flash',
                     supplied: 'mp3',
                     preload: 'metadata',
                     volume: 0.8,
                    wmode: "window",
                    useStateClassSkin: true,
                    autoBlur: false,
                    smoothPlayBar: true,
                    keyEnabled: true,
                    remainingDuration: true,
                    toggleDuration: true,
                    cssSelectorAncestor: '#' + containerId,
                    });
                    $('.jp-interface button').prop('disabled', true);
                };

                createPlayer('');
                var player = el.find('.jplayer');

                scope.$watch("path", function() {
                    if(scope.path) {
                        var src = scope.path.$$unwrapTrustedValue ? scope.path.$$unwrapTrustedValue() : scope.path;
                        var title = src.substr(src.lastIndexOf('/') + 1);
                        player.jPlayer('setMedia', {mp3: src, title: title});
                        if(scope.autoplay === "yes") {
                            player.jPlayer('play');
                        }
                        $('.jp-interface button').prop('disabled', false);
                    }
                    else {
                        $('.jp-interface button').prop('disabled', true);
                    }
                });
            };
        }
    }
})
.directive('fileModel', ['$parse', function ($parse) {
    return {
        restrict: 'A',
        link: function(scope, element, attrs) {
            var model = $parse(attrs.fileModel);
            var modelSetter = model.assign;

            element.bind('change', function(){
                scope.$parent.$apply(function(){
                    modelSetter(scope.$parent, element[0].files[0]);
                });
            });
        }
    };
}])
.directive('onlyNum', function() {
    return function(scope, element, attrs) {

        var keyCode = [8,9,37,39,48,49,50,51,52,53,54,55,56,57,96,97,98,99,100,101,102,103,104,105,110];
        element.bind("keydown", function(event) {
            if($.inArray(event.which,keyCode) == -1) {
                scope.$apply(function(){
                    scope.$eval(attrs.onlyNum);
                    event.preventDefault();
                });
                event.preventDefault();
            }

        });
    };
});
